package com.loafle.overflow.commons.dao;

import javax.persistence.EntityManager;
import javax.persistence.Persistence;
import javax.persistence.PersistenceContext;
import java.lang.reflect.ParameterizedType;

/**
 * Created by insanity on 17. 5. 25.
 */
public class JPABaseDAO<T> implements BaseDAO<T>  {

    @PersistenceContext
    private EntityManager entityManager;
    private final Class<T> entityType;

    public JPABaseDAO() {
        this.entityManager = Persistence.createEntityManagerFactory("overflow").createEntityManager();
        this.entityType = ((Class<T>) ((ParameterizedType) getClass().getGenericSuperclass()).getActualTypeArguments()[0]);
    }

    protected EntityManager getEntityManager() {
        return this.entityManager;
    }

    public T find(T entity)
    {
        return this.entityManager.find(entityType, entity);
    }

    public T create(T entity) {
        this.entityManager.getTransaction().begin();
        this.entityManager.persist(entity);
        this.entityManager.getTransaction().commit();
        return entity;
    }

    public T update(T entity) {
        this.entityManager.getTransaction().begin();
        T result = this.entityManager.merge(entity);
        this.entityManager.getTransaction().commit();
        return result;
    }

    public void delete(T entity) {
        this.entityManager.getTransaction().begin();
        this.entityManager.remove(entityManager.contains(entity) ? entity : entityManager.merge(entity));
        this.entityManager.getTransaction().commit();
    }
}
